@@ -1,4 +1,4 @@
-/* $Id: decode.c,v 1.6 2005/12/12 05:31:26 ichiro Exp $ */
+/* $Id: decode.c,v 1.12 2005/12/16 17:55:25 ichiro Exp $ */
 /*
  * Copyright (c) 2004
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
@@ -56,7 +56,7 @@ int url_decode_jis(char *dst, char *src)
 	unsigned char data[HTTPBUF + 256];
 
 	p = dst;
-	memset(data, 0, sizeof(data));
+	bzero(data, sizeof(data));
 
 	if (strlen(src) == 0) 
 		return (1);
@@ -67,6 +67,9 @@ int url_decode_jis(char *dst, char *src)
 	/* check encode method */
 	for (n = 0; n < len; n++) {
 		if (data[n] == 0x1B) { /* found JIScode */
+#ifdef DEBUG
+	printf("found JIS\n");
+#endif
 			/* printf("found JIS\n"); */
 			strcpy(p, data);
 			return (0);
@@ -75,12 +78,18 @@ int url_decode_jis(char *dst, char *src)
 		if (data[n] < 0x80) { 
 			goto loop_end;
 		} else if (data[n] >= 0x81 && data[n] <= 0x9F) {
+#ifdef DEBUG
+	printf("found SJIS\n");
+#endif
 			/* printf("found SJIS\n"); */
 			sjis2jis(p, data);
 			return (0);
 		} else if (data[n] >= 0xA1 && data[n] <= 0xDF) {
 			if ((data[n+1] < 0x80) ||
 			    (data[n+1] > 0x80 && data[n+1] < 0xA0)) {
+#ifdef DEBUG
+	printf("found SJIS+kana\n");
+#endif
 				/* printf("found SJIS+kana\n"); */
 				sjis2jis(p, data);
 				return (0);
@@ -88,7 +97,9 @@ int url_decode_jis(char *dst, char *src)
 		}
 
 		if (data[n] >= 0xF0 && data[n] <= 0xFE) {
-			/* printf("found EUC\n"); */
+#ifdef DEBUG
+	printf("found EUC\n");
+#endif
 			euc2jis(p, data);
 			return (0);
 		} else { 
@@ -382,22 +393,54 @@ char* href_conv(char* str) {
 	/* match str */
 	matched_str = (char*)malloc(matched_str_len + 1);
 	if (matched_str == NULL) return str;
-
 	*matched_str = '\0';
 	strncat(matched_str, &str[match.rm_so], matched_str_len);
 
 	/* head str */
 	head_str = (char*)malloc(head_str_len + 1);
+	if (head_str == NULL) return str;
 	*head_str = '\0';
 	strncat(head_str, &str[0], head_str_len);
 
 	/* tail str */
 	tail_str = (char*)malloc(tail_str_len + 1);
+	if (tail_str == NULL) return str;
 	*tail_str = '\0';
 	strncat(tail_str, &str[match.rm_eo], tail_str_len);
 
 	sprintf(str, "%s<a href=\"%s\">%s</a>%s",
 		head_str, matched_str, matched_str, tail_str);
 
+	/* free heap */
+	free(head_str);
+	free(matched_str);
+	free(tail_str);
+
 	return str;
 }
+
+int strcmp2(char *str1, char *str2)
+{
+	char *p, *q;
+
+	for (p = str1, q = str2; tolower(*p) == tolower(*q); p++,q++) {
+		if (*p == '\0')
+			return (0);
+	}
+	return tolower(*p) > tolower(*q) ? (1) : (-1);
+}
+
+#if 0
+int count_char(char *str)
+{
+	char *p;
+	int count = 0;
+
+	for (p = str; *p; p++) {
+		if (*p < 0x20)
+			count++;
+	}
+
+	return count;
+}
+#endif
