@@ -1,4 +1,4 @@
-/* $Id: httpd.c,v 1.27 2005/12/12 05:18:34 ichiro Exp $ */
+/* $Id: httpd.c,v 1.43 2005/12/16 17:55:25 ichiro Exp $ */
 /*
  * Copyright (c) 2004
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
@@ -86,6 +86,8 @@ void httpd_proc(struct tparam *th2, int socket)
 	char channel[C_NAME], channel2[C_NAME];
 	char uri_addr[64];
 	char base64[HTTPBUF];
+	char basic_auth[HTTPBUF];
+	char header[HTTPBUF];
 	char message[HTTPBUF];
 	char post_message[HTTPBUF + 256]; /* include url_encode mergin */
 	char post_message2[HTTPBUF + 256];
@@ -107,22 +109,23 @@ void httpd_proc(struct tparam *th2, int socket)
 
 	unsigned char *p;
 
-	/* memset zero */
-	memset(buf, 0, sizeof(buf));
-	memset(meth_name, 0, sizeof(meth_name));
-	memset(uri_addr, 0, sizeof(uri_addr));
-	memset(base64, 0, sizeof(base64));
-	memset(message, 0, sizeof(message));
-	memset(channel, 0, sizeof(channel));
-	memset(channel2, 0, sizeof(channel2));
-	memset(post_message, 0, sizeof(post_message));
-	memset(post_message2, 0, sizeof(post_message2));
-	memset(th2->postdata, 0, sizeof(th2->postdata));
+	/* bzero */
+	bzero(buf, sizeof(buf));
+	bzero(meth_name, sizeof(meth_name));
+	bzero(uri_addr, sizeof(uri_addr));
+	bzero(base64, sizeof(base64));
+	bzero(basic_auth, sizeof(basic_auth));
+	bzero(message, sizeof(message));
+	bzero(channel, sizeof(channel));
+	bzero(channel2, sizeof(channel2));
+	bzero(post_message, sizeof(post_message));
+	bzero(post_message2, sizeof(post_message2));
+	bzero(th2->postdata, sizeof(th2->postdata));
 
-	memset(cmd_cname, 0, sizeof(cmd_cname));
-	memset(cmd_cmd, 0, sizeof(cmd_cmd));
-	memset(cmd_topic, 0, sizeof(cmd_topic));
-	memset(cmd_buf, 0, sizeof(cmd_buf));
+	bzero(cmd_cname, sizeof(cmd_cname));
+	bzero(cmd_cmd, sizeof(cmd_cmd));
+	bzero(cmd_topic, sizeof(cmd_topic));
+	bzero(cmd_buf, sizeof(cmd_buf));
 
 	if (read(socket, buf, SBUFLEN) <= 0 ) {
 		printf("error: reading a request.\n");
@@ -141,67 +144,76 @@ void httpd_proc(struct tparam *th2, int socket)
 	printf("BUF(DEBUG):%s\n", buf);
 
 #endif
-	/* Cookie Authentication */
-	if ((p = strstr(buf, "Cookie:")) != NULL) {
+	/* Use cookie ? */
+	if (th2->use_cookie) {
+	   /* Cookie Authentication */
+	   if ((p = strstr(buf, "Cookie:")) != NULL) {
 		strcpy(base64, p + 8);
 #ifdef DEBUG2
 	printf("DEBUG: Cookie get.\n");
 #endif
 
-		if (sscanf(base64, "user=%[^;]; pass=%s\n",
+		if (sscanf(base64, "user=%[^;]; pass=%[^\r^\n]",
 		    cookie_user, cookie_pass) == 2);
-		else if (sscanf(base64, "pass=%[^;]; user=%s\n",
+		else if (sscanf(base64, "pass=%[^;]; user=%[^\r^\n]",
 			 cookie_pass, cookie_user) == 2);
 		else
 			printf("DEBUG:error\n");
-	}
+	    }
+	} /* Use cookie */
+
 	/* Password Authentication */
 	if ((p = strstr(buf, "Authorization: Basic")) != NULL) {
 #ifdef DEBUG2
 	printf("DEBUG: Passwd get.\n");
 #endif
 		strcpy(base64, p + 21);
-		sscanf(base64_decode(base64),
-		       "%[^:]:%s\n", web_user, web_pass);
+		sscanf(base64, "%[^\r^\n]", basic_auth);
+		sscanf(base64_decode(basic_auth),
+		       "%[^:]:%[^\n]", web_user, web_pass);
 	}
 
-	if ((strcmp(cookie_user, th2->auth_user) == 0) &&
-	    (strcmp(cookie_pass, th2->auth_pass) == 0)) {
-		/* send header HTTP/1.1 200 OK */
-		send_msg(socket, http_header0);
+	if (th2->use_cookie &&
+	   (strcmp(cookie_user, th2->auth_user) == 0) &&
+	   (strcmp(cookie_pass, th2->auth_pass) == 0)) {
 #ifdef DEBUG2
 	printf("DEBUG: Cookie Authentication done.\n");
 #endif
+		/* copy header */
+		strcpy(header, http_header0);
 	} else if ((strcmp(web_user, th2->auth_user) == 0) &&
 		   (strcmp(web_pass, th2->auth_pass) == 0)) {
-		/* send header HTTP/1.1 200 OK */
-		send_msg(socket, http_header0);
 #ifdef DEBUG2
 	printf("DEBUG: Passwd Authentication done.\n");
 #endif
-		/* send cookie info */
+		/* copy header */
+		strcpy(header, http_header0);
+
+		/* make cookie info */
 		timer = time(NULL) + th2->cookie_expire * 24 * 60 * 60;
 		date = gmtime(&timer);
 		strftime(date_str, 255, "%a, %d-%b-%Y %H:%M:%S GMT;\n", date);
+
 		/* user */
 		sprintf(message, "Set-Cookie: user=%s; expires=", web_user);
 		strcat(message, date_str);
-		send_msg(socket, message);
+		strcat(header, message);
+
 		/* pass */
 		sprintf(message, "Set-Cookie: pass=%s; expires=",web_pass);
 		strcat(message, date_str);
-                send_msg(socket, message);
+		strcat(header, message);
 	} else {
 		/* cannot auth */
-		send_msg(socket, http_auth_401_header);
+		send_msg(socket, th2, http_auth_401_header);
 		return;
 	}
 
 	/* check and clear auth info */
-	memset(web_user, 0, sizeof(web_user));
-	memset(web_pass, 0, sizeof(web_pass));
-	memset(cookie_user, 0, sizeof(cookie_user));
-	memset(cookie_pass, 0, sizeof(cookie_pass));
+	bzero(web_user, sizeof(web_user));
+	bzero(web_pass, sizeof(web_pass));
+	bzero(cookie_user, sizeof(cookie_user));
+	bzero(cookie_pass, sizeof(cookie_pass));
 
 	/* POST session */
 	if ((strcmp(meth_name, "POST")) == 0) {
@@ -219,8 +231,14 @@ void httpd_proc(struct tparam *th2, int socket)
 			post_length = atoi(s + 16);
 
 		/* if cannot read 'm=' or 'cname=', read more */
-		if((!strstr(buf, "m=")) && (!strstr(buf, "cname=")))
+		if((!strstr(buf, "m=")) && (!strstr(buf, "cname="))) {
+			/* zero clear buffer */
+			bzero(buf, sizeof(buf));
 			read(socket, buf, post_length);
+#ifdef DEBUG
+			printf("%s\n", buf);
+#endif
+		}
 
 		/* Get post message */
 		if ((s = strstr(buf, "m="))) {
@@ -235,10 +253,10 @@ void httpd_proc(struct tparam *th2, int socket)
 			}
 
 			/* URL decord */
+			url_decode_jis(post_message2, post_message);
 #ifdef DEBUG
 			printf("decode: %s\n", post_message);
 #endif
-			url_decode_jis(post_message2, post_message);
 
 			/* Get channel number to post */
 			sscanf(uri_addr + 1, "channel%d",
@@ -254,15 +272,24 @@ void httpd_proc(struct tparam *th2, int socket)
 				 th2->real_nickname, channel2,
 				 post_message2);
 
+			/* store own messages */
+			strcpy(th2->l_buff, th2->postdata);
+			priv_recv(th2);
+			bzero(th2->l_buff, sizeof(th2->l_buff));
+
+			/* show only own throw message */
+			update = 1;
+			th2->read_count[post_ch_number]--;
+
 			/* clear trans. empty flag */
 			th2->t_empty = 0;
 			th2->t_lock = 0;
 
 		} else if ((s = strstr(buf, "cname="))) {
 			/* clear buffer */
-			memset(cmd_cname, 0, sizeof(cmd_cname));        
-			memset(cmd_cmd, 0, sizeof(cmd_cmd));    
-			memset(cmd_topic, 0, sizeof(cmd_topic));
+			bzero(cmd_cname, sizeof(cmd_cname));        
+			bzero(cmd_cmd, sizeof(cmd_cmd));    
+			bzero(cmd_topic, sizeof(cmd_topic));
 
 			if (post_length < 22) {
 				/* clear trans. empty flag */
@@ -276,17 +303,19 @@ void httpd_proc(struct tparam *th2, int socket)
 
 			/* cname=&cname2=&cmd=part&topic= */
 			q = strtok(post_message, "&");
-			if (q != NULL) strcpy(cmd_cname, q + 6);	/* 6: cname= */
+			if (q != NULL) strcpy(cmd_cname, q + 6);
+							/* 6: cname= */
 
 			q = strtok(NULL, "&");
 			if (q != NULL) {
 				if (strlen(cmd_cname) == 0)
-					strcpy(cmd_cname, q + 7);	/* 7: cname2= */
+					strcpy(cmd_cname, q + 7);
+							/* 7: cname2= */
 			}
 
 			q = strtok(NULL, "&");
 			if (q != NULL) {
-				strcpy(cmd_cmd, q + 4);		/* 4: cmd= */
+				strcpy(cmd_cmd, q + 4);	/* 4: cmd= */
 			}
 
 			q = strtok(NULL, "&");
@@ -294,33 +323,38 @@ void httpd_proc(struct tparam *th2, int socket)
 				strcpy(cmd_buf, q + 6); /* 6: topic= */
 			}
 #ifdef DEBUG
-			printf("post_message=%s\n", post_message);
 			printf("cname=%s\ncmd=%s\ntopic=%s\n",
 				cmd_cname, cmd_cmd, cmd_buf);
 #endif
 			/* make command message */
 			if (strcmp(cmd_cmd, "join") == 0) {
+			   if(strlen(cmd_cname) > 0){
 				url_decode_rawdata(channel, cmd_cname);
-				channel_decode(channel2, channel);
-				sprintf(th2->postdata, ":%s JOIN %s\r\n",
-					th2->real_nickname,
-					channel2);
+				/* send join */
+				send_join(th2, channel);
+			   }
 			} else if (strcmp(cmd_cmd, "part") == 0) {
+			   if(strlen(cmd_cname) > 0){
 				url_decode_rawdata(channel, cmd_cname);
-				channel_decode(channel2, channel);
-				sprintf(th2->postdata, ":%s PART %s\r\n",
-					th2->real_nickname,
-					channel2);
+				/* send part */
+				send_part(th2, channel);
+			   }
 			} else if (strcmp(cmd_cmd, "nick") == 0) {
+			   if(strlen(cmd_buf) > 0){
 				sprintf(th2->postdata, ":%s NICK %s\r\n",
 					th2->real_nickname, cmd_buf);
+			   }
 			} else if (strcmp(cmd_cmd, "topic") == 0) {
+			   if((strlen(cmd_cname) > 0) &&
+				(strlen(cmd_buf) > 0)){
 				url_decode_rawdata(channel, cmd_cname);
+				/* any -> #hoge*.jp */
 				channel_decode(channel2, channel);
 				url_decode_jis(cmd_topic, cmd_buf);
 				sprintf(th2->postdata, ":%s TOPIC %s :%s\r\n",
 					th2->real_nickname,
 					channel2, cmd_topic);
+			   }
 			} else {
 				goto command_start;
 			}
@@ -331,6 +365,7 @@ void httpd_proc(struct tparam *th2, int socket)
 		th2->t_lock = 0;
 	}	/* POST */
 
+#if 0
 	/* favicon.ico */
 	if (strstr(uri_addr + 1, "favicon.ico")) {
 #ifdef DEBUG
@@ -354,22 +389,20 @@ void httpd_proc(struct tparam *th2, int socket)
 
 		close(r_fd);
 		send_msg(socket, http_footer);
-		return;
 	} /* favicon.ico */
+#endif
 
 	/* start COMMAND page */
 command_start:
-	if ((strcmp(uri_addr + 1, "cmd")) == 0) {
+	if ((strcmp(uri_addr + 1, "cmd") == 0) &&
+	    (strcmp(meth_name, "GET") == 0)) {
 #ifdef DEBUG
 		printf("COMMAND page\n");
 #endif
-		/* send header */
-		send_msg(socket, http_header1);
-		send_msg(socket, http_header2);
-
 		print_command_page(th2, socket);
 
-		send_msg(socket, http_footer);
+		/* send data */
+		send_msg(socket, th2, header);
 
 		return;
 	}
@@ -380,10 +413,6 @@ index_start:
 #ifdef DEBUG
 		printf("Sub index session\n");
 #endif
-		/* send header */
-		send_msg(socket, http_header1);
-		send_msg(socket, http_header2);
-
 		/* check update or not */
 		if (number >= 50) {
 			update = 1;
@@ -394,26 +423,25 @@ index_start:
 		if (strlen(th2->topic[number]) > 0) {
 			sprintf(message, "$B$H$T$/(B :%s<br>",
 				th2->topic[number]);
-			send_msg(socket, message);
+			send_buf(th2, message);
 		}
 
 		/* print NAMES */
 		if (strlen(th2->member_names[number]) > 0) {
 			sprintf(message, "users:%s<br>",
 				th2->member_names[number]);
-			send_msg(socket, message);
+			send_buf(th2, message);
 		}
 
 		/* print FORM */
 		sprintf(message,
-			"<form action=\"/channel%d\" method=\"post\">",
-			number);
-		send_msg(socket, message);
-		send_msg(socket,
+			"<form action=\"/channel%d\" method=\"post\">", number);
+		send_buf(th2, message);
+		send_buf(th2,
 			 "<input type=text name=\"m\" size=\"10\">");
-		send_msg(socket,
+		send_buf(th2,
 			 "<input type=submit value=\" $BAw?.(B \"></form>");
-		send_msg(socket,
+		send_buf(th2,
 			"<a accesskey=\"8\" href=\"/\">back[8]</a><hr>");
 
 		if (update)
@@ -426,18 +454,19 @@ index_start:
 			     n < th2->history_count[number]; n++) {
 				sprintf(message, "%s",
 					th2->data[number][n]);
-				send_msg(socket, message);
+				send_buf(th2, message);
 			}
 		} else {
 			for (n = th2->history_count[number] - 1;
 			     n >= start_number; n--) {
 				sprintf(message, "%s",
 					th2->data[number][n]);
-				send_msg(socket, message);
+				send_buf(th2, message);
 			}
 		}
 		th2->read_count[number] = th2->history_count[number];
-		send_msg(socket, http_footer);
+
+		send_msg(socket, th2, header);
 
 		update = 0;
 
@@ -445,51 +474,119 @@ index_start:
 	}
 
 	/* start main index */
-	/* send header */
-	send_msg(socket, http_header1);
-	send_msg(socket, http_header2);
-
+#ifdef DEBUG
+	printf("start main index\n");
+#endif
 	for (n = 0; n <= th2->ch_count; n++) {
 	    if (n < 9) {
 	       sprintf(message,
-		   "%d <a accesskey=\"%d\" href=\"channel%d\">%s</a> \
-		    <a href=\"channel%d\">(%d)</a><br>",
+		   "%d <a accesskey=\"%d\" href=\"channel%d\">%s</a>"
+		   "<a href=\"channel%d\">(%d)</a><br>",
 		    n, n, n, th2->name[n],
 		    n + 50, th2->history_count[n] -
 		    th2->read_count[n]);
 	    } else {
 	sprintf(message,
-		   "* <a href=\"channel%d\">%s</a> \
-		    <a href=\"channel%d\">(%d)</a><br>",
+		   "* <a href=\"channel%d\">%s</a>"
+		   "<a href=\"channel%d\">(%d)</a><br>",
 		    n, th2->name[n],
 		    n + 50, th2->history_count[n] -
 		    th2->read_count[n]); 	
 	    }
-		send_msg(socket, message);
+		send_buf(th2, message);
 	}
 	sprintf(message,
-		"<hr>9 <a accesskey=9 href=\"cmd\"> \
-			command page[9]</a><br>");
-	send_msg(socket, message);
+		"<hr>9 <a accesskey=9 href=\"cmd\">"
+		"command page[9]</a><br>");
+	send_buf(th2, message);
 	sprintf(message,
 		"<br><left>kircd Ver %s </left>", VERSION);
-	send_msg(socket, message);
-	send_msg(socket, http_footer);
+	send_buf(th2, message);
+
+	send_msg(socket, th2, header);
 
 	return;
 }
 
-
-int send_msg(int fd, char *msg)
+void send_buf(struct tparam *th, char *addmsg)
 {
-	int n;
-	n = strlen(msg);
+	int buf_len, add_len;
 
-	if (write(fd, msg, n) != n){
-		printf("error: writing.");
+	buf_len = strlen(th->send_buf);
+	add_len = strlen(addmsg);
+
+	if (th->send_buflen < (buf_len + add_len + 1)) {
+#ifdef DEBUG
+		printf("SEND_BUF %d -> ", th->send_buf); 
+#endif
+		th->send_buflen = th->send_buflen + HTTPBUF;
+		th->send_buf = (char *)realloc(th->send_buf,
+				th->send_buflen);
+#ifdef DEBUG
+		printf("%d\n", th->send_buf); 
+#endif
 	}
 
-	return n;
+	strcat(th->send_buf, addmsg);
+}
+
+int send_msg(int fd, struct tparam *th, char *header_msg)
+{
+	int n;
+	int header_len, msg_len, footer_len, total_len;
+	int buffer_len;
+	char contentlen[30];
+	char *buffer;
+
+	header_len = strlen(http_header1);
+	msg_len    = strlen(th->send_buf);
+	footer_len = strlen(http_footer);
+
+	total_len  = header_len + msg_len + footer_len;
+
+	/* make Content-Length */
+	sprintf(contentlen,
+		"Content-Length: %d\r\n\r\n", total_len);
+
+	/* memory allocate */
+	buffer = (char*)malloc(total_len + 1);
+	*buffer = '\0';
+
+#ifdef DEBUG
+	printf("send_msg: length = %d\r\n\r\n",
+		header_len + msg_len + footer_len);
+#endif
+	/* copy control header */
+	strcat(buffer, header_msg);
+	strcat(buffer, http_contype);
+	strcat(buffer, contentlen);
+
+	/* copy html header */
+	strcat(buffer, http_header1);
+
+	/* copy message */
+	strcat(buffer, th->send_buf);
+
+	/* copy footer */
+	strcat(buffer, http_footer);
+
+	buffer_len = strlen(buffer);
+
+#ifdef DEBUG
+	printf("SEND_DATA:(%d)byte\n%s\n", buffer_len, buffer);
+#endif
+
+	if (write(fd, buffer, buffer_len) != buffer_len){
+		printf("error: writing %d chars\n", buffer_len);
+	}
+
+	/* clear buffer */
+	strcpy(th->send_buf, "\0");
+
+	/* free buffer */
+	free(buffer);
+
+	return buffer_len;
 }
 
 void print_command_page(struct tparam *th2, int socket)
@@ -497,38 +594,89 @@ void print_command_page(struct tparam *th2, int socket)
 	char str[C_NAME];
 	int i;
 
-	send_msg(socket,
+	send_buf(th2,
 	   "<form action=\"/cmd\" method=\"post\">");
-	send_msg(socket,
-	   "<input type=\"submit\" \
-		accesskey=\"1\" value=\"OK[1]\">");
-	send_msg(socket,
-	   "<a accesskey=\"8\" href=\"/\"> back[8]</a><hr>");
 	
-	send_msg(socket, "Channel:");
-	send_msg(socket,
+	send_buf(th2, "Channel:");
+	send_buf(th2,
 	   "<br><select name=\"cname\">");
-	send_msg(socket,
+	send_buf(th2,
 	   "<option value=\"\">specific</option>");
 	for (i = 0; i < th2->ch_count + 1; i++) {
 		sprintf(str, "<option value=\"%s\">%s</option>",
 			th2->name[i], th2->name[i]);
-		send_msg(socket, str);
+		send_buf(th2, str);
 	}
-	send_msg(socket, "</select>");
+	send_buf(th2, "</select>");
 
-	send_msg(socket,
+	send_buf(th2,
 	   "<br><input type=text name=\"cname2\" size=\"17\">");
 
-	send_msg(socket,
+	send_buf(th2,
 	   "<br><input type=radio name=\"cmd\" value=\"part\">PART");
-	send_msg(socket,
+	send_buf(th2,
 	   "<input type=radio name=\"cmd\" value=\"join\">JOIN");
 
-	send_msg(socket,
+	send_buf(th2,
 	   "<hr><input type=radio name=\"cmd\" value=\"topic\">TOPIC");
-	send_msg(socket,
+	send_buf(th2,
 	   "<input type=radio name=\"cmd\" value=\"nick\">NICK");
-	send_msg(socket,
-	   "<br><input type=text name=\"topic\" size=\"17\"></form>");
+	send_buf(th2,
+	   "<br><input type=text name=\"topic\" size=\"17\">");
+
+	send_buf(th2,
+	   "<hr><input type=submit value=\" $BAw?.(B [1]\">");
+	send_buf(th2,
+	   "<a accesskey=\"8\" href=\"/\"> back[8]</a></form>");
+}
+
+int send_join(struct tparam *th, char *channel)
+{
+	int i;
+	char channel2[C_NAME];
+
+	/* any ->%hoge */
+	channel_encode(channel2, channel);
+
+	/* search exist channel */
+	for (i = 0; i <= th->ch_count; i++) {
+		if (strcmp2(th->name[i], channel2) == 0) {
+#ifdef DEBUG
+			printf("channel exist!\n");
+#endif
+			return (0);
+		}
+	}
+
+	/* %hoge -> #hoge*.jp */
+	channel_decode(channel2, channel);
+
+	if (channel2[0] == '#') { /* make channel */
+		sprintf(th->postdata, ":%s JOIN %s\r\n",
+			th->real_nickname, channel2);
+	} else {			/* make channel with person */
+		/* make new channel */
+		channel_make(th, channel2);
+	}
+
+	return (0);
+
+}
+
+int send_part(struct tparam *th, char *channel)
+{
+	char channel2[C_NAME];
+
+	/* any -> #hoge*.jp */
+	channel_decode(channel2, channel);
+
+	if (channel2[0] == '#') {	/* disconnect from irc-channel */
+		sprintf(th->postdata, ":%s PART %s\r\n",
+			th->real_nickname, channel2);
+	} else {			/* disconnect from person */
+		/* delete channel */
+		channel_delete(th, channel2);
+	}
+
+	return (0);
 }
