@@ -1,5 +1,6 @@
-/* $Id: kircd.h,v 1.16 2005/12/11 21:26:00 ichiro Exp $ */
+/* $Id: kircd.h,v 1.22 2005/12/16 17:55:25 ichiro Exp $ */
 /* #define DEBUG */
+/* #define DEBUG2 */
 /*
  * Copyright (c) 2004
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
@@ -37,8 +38,11 @@
 #include "config.h"
 #endif
 
-#include <glib/gthread.h>
+#if !HAVE_BZERO && HAVE_MEMSET
+#define bzero(buf, bytes)  ((void)memset(buf, 0, bytes))
+#endif
 
+#include <glib/gthread.h>
 
 #define CLIENT 0
 #define SERVER 1
@@ -59,8 +63,7 @@
 #define NAME_LEN 10
 
 /* rfc1459, max length of IRC message is 512 byte */
-/* #define IRC_MAX 515 */
-#define IRC_MAX 1024
+#define IRC_MAX 515
 
 struct tparam {
 	int socket_number;
@@ -78,6 +81,7 @@ struct tparam {
 
 	int max_line;
 	int line_reverse;
+	int use_cookie;
 	int cookie_expire;
 	char name[CH_MAX][C_NAME];
 	int  ch_count;
@@ -95,7 +99,12 @@ struct tparam {
 	/* auth (copy) */
 	char auth_user[40];
 	char auth_pass[40];
+
+	/* send buffer */
+	char *send_buf;
+	int  send_buflen;
 };
+
 struct config {
 	char hostname[128];
 	char server_port[40];
@@ -109,6 +118,7 @@ struct config {
 	char auth_pass[40];
 	int max_line;
 	int line_reverse;
+	int use_cookie;
 	int cookie_expire;
 };
 
@@ -119,18 +129,25 @@ gpointer watch(gpointer);
 gpointer lineget(gpointer);
 extern gpointer httpd(gpointer);
 
-extern int send_msg(int, char *);
+extern int send_postdata(int, char *);
+extern int send_msg(int, struct tparam *, char *);
+extern void send_buf(struct tparam *, char *);
 extern int readconf(unsigned char *, struct config *);
 extern int keitai_start(struct tparam *);
 extern int priv_recv(struct tparam *);
 extern int topic_recv(struct tparam *);
 extern int part_recv(struct tparam *);
-extern int join_recv(struct tparam *, char *);
+extern int channel_delete(struct tparam *, char *);
+extern int join_recv(struct tparam *);
+extern int channel_make(struct tparam *, char *);
 extern int names_recv(struct tparam *);
 extern int ping_recv(struct tparam *);
 extern int send_names(struct tparam *, char *);
 extern int nick_recv(struct tparam *);
 
+extern int send_join(struct tparam *, char *);
+extern int send_part(struct tparam *, char *);
+
 extern int url_decode_jis(char *, char *);
 extern int url_decode_rawdata(char *, char *);
 extern int sjis2jis(char *, char *);
@@ -140,3 +157,7 @@ extern int channel_decode(char *, char *);
 extern int ltgt_encode(char *, char *);
 extern char * base64_decode(char *);
 extern char * href_conv(char *);
+extern int strcmp2(char *, char *);
+#if 0
+extern int count_char(char *);
+#endif
