@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.8 2005/12/12 03:45:30 ichiro Exp $ */
+/* $Id: main.c,v 1.13 2005/12/16 17:55:25 ichiro Exp $ */
 /*
  * Copyright (c) 2004
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
@@ -117,7 +117,7 @@ int main(int argc, char *argv[])
 		printf("error: Configration\n");
 
 	/* make socket for IRC client (IPv4/v6) */
-	memset(&h, 0, sizeof(h));
+	bzero(&h, sizeof(h));
 	h.ai_family = PF_UNSPEC;
 	h.ai_socktype = SOCK_STREAM;
 	s1 = get_addr(h, cf->hostname, cf->server_port, CLIENT);
@@ -127,7 +127,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* make socket for HTTP server (IPv4 only) */
-	memset(&h, 0, sizeof(h));
+	bzero(&h, sizeof(h));
 	h.ai_family = AF_INET;
 	h.ai_socktype = SOCK_STREAM;
 	h.ai_flags = AI_PASSIVE;
@@ -136,7 +136,7 @@ int main(int argc, char *argv[])
 		printf("error: faild to make socket for HTTP server\n");
 		exit(1);
 	}
-	listen(s2, 1);
+	listen(s2, 10);
 
 	/* make thread for IRC client */
 	th.socket_number = s1;
@@ -152,6 +152,7 @@ int main(int argc, char *argv[])
 	/* copy config option */
 	th.max_line = cf->max_line;
 	th.line_reverse = cf->line_reverse;
+	th.use_cookie = cf->use_cookie;
 	th.cookie_expire = cf->cookie_expire;
 	strcpy(th.real_nickname, cf->nickname);
 
@@ -159,6 +160,11 @@ int main(int argc, char *argv[])
 	strcpy(th.auth_user, cf->auth_user);
 	strcpy(th.auth_pass, cf->auth_pass);
 
+	/* initialize memory allocate for send_buf */
+	th.send_buf = (char*)malloc(HTTPBUF * 2);
+	th.send_buflen = HTTPBUF * 2;
+	*th.send_buf = '\0';
+
 	/* initialize gthread */
 	g_thread_init(NULL);
 
@@ -190,20 +196,33 @@ void init(int s, struct config *cf)
 		strcpy(p, "PASS ");
 		strcat(p, cf->password);
 		strcat(p, "\r\n");
-		send_msg(s, p);
+		send_postdata(s, p);
 	}
 
 	strcpy(p, "USER ");
 	strcat(p, cf->username);
 	strcat(p, " \"\" \"\" \"\"\r\n");
-	send_msg(s, p);
+	send_postdata(s, p);
 
 	strcpy(p, "NICK ");
 	strcat(p, cf->nickname);
 	strcat(p, "\r\n");
-	send_msg(s, p);
+	send_postdata(s, p);
 }
 
+int send_postdata(int fd, char *msg)
+{
+	int n;
+	n = strlen(msg);
+
+	if (write(fd, msg, n) != n){
+		printf("error: writing %d chars\n", n);
+	}
+
+	return n;
+}
+	
+
 gpointer
 watch(gpointer param)
 {
@@ -241,10 +260,8 @@ watch(gpointer param)
 			printf("send: %s\n",th->postdata);
 #endif
 			/* SEND DATA */
-			send_msg(s, th->postdata);
-			strcpy(th->l_buff, th->postdata);
-			memset(th->l_buff, 0, sizeof(th->l_buff));
-			memset(th->postdata, 0, sizeof(th->postdata));
+			send_postdata(s, th->postdata);
+			bzero(th->postdata, sizeof(th->postdata));
 
 			th->t_empty = 1; /* set empty flag */
 		}
